/*
* Jalview - A Sequence Alignment Editor and Viewer
* Copyright (C) 2005 AM Waterhouse, J Procter, G Barton, M Clamp, S Searle
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
*/
package jalview.analysis;

import jalview.analysis.*;

import jalview.datamodel.*;

import java.util.*;


/**
 * Takes in a vector of sequences and column start and column end
 * and returns a vector of size (end-start+1). Each element of the
 * vector contains a hashtable with the keys being residues and
 * the values being the count of each residue in that column.
 * This class is used extensively in calculating alignment colourschemes
 * that depend on the amount of conservation in each alignment column.
 * @author $author$
 * @version $Revision$
 */
public class AAFrequency
{
    /** Takes in a vector of sequences and column start and column end
    * and returns a vector of size (end-start+1). Each element of the
    * vector contains a hashtable with the keys being residues and
    * the values being the count of each residue in that column.
    * This class is used extensively in calculating alignment colourschemes
    * that depend on the amount of conservation in each alignment column. */
    public static Vector calculate(Vector sequences, int start, int end)
    {
        Vector result = new Vector();

        for (int i = start; i <= end; i++)
        {
            Hashtable residueHash = new Hashtable();
            int maxCount = 0;
            String maxResidue = "-";
            int nongap = 0;

            for (int j = 0; j < sequences.size(); j++)
            {
                if (sequences.elementAt(j) instanceof Sequence)
                {
                    Sequence s = (Sequence) sequences.elementAt(j);

                    if (s.getSequence().length() > i)
                    {
                        String res = s.getSequence().charAt(i) + "";

                        if (!jalview.util.Comparison.isGap(res.charAt(0)))
                        {
                            nongap++;
                        }
                        else
                        {
                            res = "-"; // we always use this for gaps in the property vectors
                        }

                        if (residueHash.containsKey(res))
                        {
                            int count = ((Integer) residueHash.get(res)).intValue();
                            count++;

                            if (!jalview.util.Comparison.isGap(res.charAt(0)) &&
                                    (count >= maxCount))
                            {
                                if (count > maxCount)
                                {
                                    maxResidue = res;
                                }
                                else if (maxResidue.indexOf(res) == -1)
                                {
                                    maxResidue += res;
                                }

                                maxCount = count;
                            }

                            residueHash.put(res, new Integer(count));
                        }
                        else
                        {
                            residueHash.put(res, new Integer(1));
                        }
                    }
                    else
                    {
                        if (residueHash.containsKey("-"))
                        {
                            int count = ((Integer) residueHash.get("-")).intValue();
                            count++;
                            residueHash.put("-", new Integer(count));
                        }
                        else
                        {
                            residueHash.put("-", new Integer(1));
                        }
                    }
                }
            }

            residueHash.put("maxCount", new Integer(maxCount));

            if (maxCount < 0)
            {
                System.out.println("asasa " + maxCount);
            }

            residueHash.put("maxResidue", maxResidue);
            residueHash.put("size", new Integer(sequences.size()));
            residueHash.put("nongap", new Integer(nongap));
            result.addElement(residueHash);
        }

        return result;
    }
}
